
#ifndef _CSF_H
  #define _CSF_H

  #ifdef __cplusplus
  extern "C" {
  #endif

  #include <stdlib.h>

  #include "pal.h"
  #include "car.h"

  #define CSF_HEADERSIZE 512
  #define CSF_DESCSIZE 64
  #define CSF_LABELSIZE 192

  #define CSF_NOCOLORS 0

  #define CSF_ID 0x46534343
  #define CSF_DESC "Cybertek Sprite/Font"
  #define CSF_VERSION 0x00010000

  typedef struct
  {
    char     desc [CSF_DESCSIZE];     // Description (should be FONT_DESC)
    char     label [CSF_LABELSIZE];   // user defined label
    long     id;                    // Type of the Font
    long     version;
    long     bytesPerPixel;           // Number of bytes per pixel
    long     sizeX;                   // Horizontal character length
    long     sizeY;                   // Vertical character length
    long     size;                    // it's just sizeX*sizeY*bytesPerPixel
    long     baseColor;               // First Color used
    long     maxColor;                // Number of Colors used
    long     count;                   // total count of characters

    long     maxChar;                 // Number of defined characters in font
    long     xSpacing;                // number of free pixels beetween each column
    long     ySpacing;                // numvber of free pixel beetwwn each line
				      // == character not defined

    long     *charTable;         // pointer to character offsets table -1 (user must create)
    TRGBColor *colors;       // Pointer to Colors (user must create)
    char     *raw;                    // Pointer to character pixels (user must create)

    char     reserved [CSF_HEADERSIZE
			-CSF_DESCSIZE*sizeof (char)
			-CSF_LABELSIZE*sizeof (char)
			-12*sizeof (long)

			-sizeof (void *)
			-sizeof (void *)
			-sizeof (void *)];

    // now goes Characters Table [256] the local colors, the raw data
  } TCSF;

  extern  TCSF *CSF_new (TCAR *carHandle,const char *spriteName);
  extern  TCSF *CSF_dup (TCSF *csf);

  extern void CSF_delete ( TCSF *spr);
  extern void CSF_setPal ( TCSF *spr, TRGBColor *colors, int firstColor, int numColors);
  extern TCSF *CSF_create (void *rawBuf, int sizeX, int sizeY,
				  TRGBColor *palette, int count,
				  long charTable [256]);

  extern int CSF_changeBaseColor ( TCSF *spr, long newBaseColor);

  extern int CSF_save ( TCSF *spr, const char *fileName);

  extern int CSF_updateColors ( TCSF *spr,  TRGBColor *pal,
				  int baseColor, int palLen); // = 2, = 254

  extern void *CSF_raw (TCSF *spr, int n);  // = 0
  extern int CSF_changeBaseColor (TCSF *spr, long newBaseColor);
  extern void CSF_cpy (TCSF *spr, void *Dst,
		   int Dst_X, int Dst_Y, int Dst_sizeX, int Dst_sizeY, int n); // = 0

  extern int CSF_addRaw (char *draw, int dsizeX, int dsizeY,
			 char *sraw, int ssizeX, int ssizeY, int one);

  extern TCSF *CSF_createMulti (TCSF **sprList,
				int sizeX, int sizeY,
				long charTable [256]);

  extern void CSF_put (TCSF *csf, int x, int y, int n); // = 0

  #define CSF_VIEW_CENTER 9999
  int CSF_view (TCAR *carHandle, const char *fileName, TRGBColor pal [256],
                int n, int x, int y); // = CSF_VIEW_CENTER, = CSF_VIEW_CENTER

  #include "csffont_.h"

  #ifdef __cplusplus
  }
  #endif

#endif
