
#ifndef _KEYB_H
  #define _KEYB_H

  #ifdef __cplusplus
  extern "C" {
  #endif


  typedef struct
  {
    unsigned int scrollLock : 1;
    unsigned int numLock : 1;
    unsigned int capsLock : 1;
    unsigned int break_ : 1;
  } TKEYB_flags;

  extern char KEYB_defaultCtrlBreakMessage [];
  extern char *KEYB_ctrlBreakMessage;

  extern TKEYB_flags KEYB_flags;
  extern int KEYB_enableLEDS;
  extern int KEYB_releaseShift;
  extern int KEYB_enableBreak;
  extern int KEYB_enableReboot;
  extern int KEYB_updateLock;

  extern void KEYB_init (void);
  // Initializes keyboard library. Use before any other functions.
  // Installs new keyboard interrupt handler.

  extern void KEYB_done (void);
  // Deinitialized keyboard library. Use before terminating your
  // application !

  extern int  KEYB_pressed (char scanCode);
  // Returns a non-zero if key with ScanCode is being hold down. For
  // available keys see KEY_XXXXX defines in the KEYB.H file

  extern int  KEYB_hit (char scanCode);
  // Returns a non-zero if the key with ScanCode has been hit or it is held down.

  extern int KEYB_get (void);
  // If a key is pressed returns its ScanCode else returns zero. Returns
  // LAST pressed key from keyboard buffer

  extern void KEYB_flush (void);
  // clears keyboard buffer

  extern int KEYB_wait (void);
  // If a key is pressed returns its ScanCode else waits until one is

  extern char KEYB_ascii (char scanCode);
  // converts scancode to an ascii character

  extern char KEYB_key (char scanCode);
  // converts scancode to scancode (checks num lock)

//  extern void KEYB_setWaitHandler ( int (*handler) (void));

  extern char *KEYB_desc (char scanCode);

  extern void KEYB_update (void);

  extern void KEYB_keyBufReset (void);

  #define KEYB_lshift (KEYB_pressed (KEY_LSHIFT) != 0)
  #define KEYB_rshift (KEYB_pressed (KEY_RSHIFT) != 0)
  #define KEYB_shift (KEYB_lshift || KEYB_rshift)

  #define KEYB_lalt (KEYB_pressed (KEY_LALT) != 0)
  #define KEYB_ralt (KEYB_pressed (KEY_RALT ) != 0)
  #define KEYB_alt (KEYB_lalt || KEYB_ralt)

  #define KEYB_lctrl (KEYB_pressed (KEY_LCTRL) != 0)
  #define KEYB_rctrl (KEYB_pressed (KEY_RCTRL ) != 0)
  #define KEYB_ctrl (KEYB_lctrl || KEYB_rctrl)

  #define KEYB_setDirectMode KEYB_init
  #define KEYB_setBIOSMode KEYB_done

  #define KEYB_shifted ( (KEYB_shift && (!KEYB_flags.capsLock) ) || ((!KEYB_shift) && KEYB_flags.capsLock))

  #define KEY_PRESSED             (1)
  #define KEY_HIT                 (2)
  #define KEY_HOLD (KEY_HIT | KEY_PRESSED)

  #define KEY_A                   (0x1E)
  #define KEY_B                   (0x30)
  #define KEY_C                   (0x2E)
  #define KEY_D                   (0x20)
  #define KEY_E                   (0x12)
  #define KEY_F                   (0x21)
  #define KEY_G                   (0x22)
  #define KEY_H                   (0x23)
  #define KEY_I                   (0x17)
  #define KEY_J                   (0x24)
  #define KEY_K                   (0x25)
  #define KEY_L                   (0x26)
  #define KEY_M                   (0x32)
  #define KEY_N                   (0x31)
  #define KEY_O                   (0x18)
  #define KEY_P                   (0x19)
  #define KEY_Q                   (0x10)
  #define KEY_R                   (0x13)
  #define KEY_S                   (0x1F)
  #define KEY_T                   (0x14)
  #define KEY_U                   (0x16)
  #define KEY_V                   (0x2F)
  #define KEY_W                   (0x11)
  #define KEY_X                   (0x2D)
  #define KEY_Y                   (0x15)
  #define KEY_Z                   (0x2C)

  #define KEY__0                   (0x52)
  #define KEY__1                   (0x4F)
  #define KEY__2                   (0x50)
  #define KEY__3                   (0x51)
  #define KEY__4                   (0x4B)
  #define KEY__5                   (0x4C)
  #define KEY__6                   (0x4D)
  #define KEY__7                   (0x47)
  #define KEY__8                   (0x48)
  #define KEY__9                   (0x49)
  #define KEY__PERIOD              (0x53)
  #define KEY__MINUS               (0x4A)
  #define KEY__PLUS                (0x4E)
  #define KEY__STAR                (0x37)

  #define KEY_1                  (0x02)
  #define KEY_2                  (0x03)
  #define KEY_3                  (0x04)
  #define KEY_4                  (0x05)
  #define KEY_5                  (0x06)
  #define KEY_6                  (0x07)
  #define KEY_7                  (0x08)
  #define KEY_8                  (0x09)
  #define KEY_9                  (0x0a)
  #define KEY_0                  (0x0b)

  #define KEY_MINUS               (0x0C)  /* -_ */
  #define KEY_PLUS                (0x0D)  /* =+ */
  #define KEY_LBRACKET            (0x1A)  /* [{ */
  #define KEY_RBRACKET            (0x1B)  /* ]} */
  #define KEY_SEMICOLON           (0x27)  /* ;: */
  #define KEY_RQUOTE              (0x28)  /* '" */
  #define KEY_LQUOTE              (0x29)  /* `~ */
  #define KEY_PERIOD              (0x33)  /* ,< */
  #define KEY_COMMA               (0x34)  /* .> */
  #define KEY_SLASH               (0x35)  /* /? */
  #define KEY_BACKSLASH           (0x2B)  /* \| */
  #define KEY_F1                  (0x3B)
  #define KEY_F2                  (0x3C)
  #define KEY_F3                  (0x3D)
  #define KEY_F4                  (0x3E)
  #define KEY_F5                  (0x3F)
  #define KEY_F6                  (0x40)
  #define KEY_F7                  (0x41)
  #define KEY_F8                  (0x42)
  #define KEY_F9                  (0x43)
  #define KEY_F10                 (0x44)
  #define KEY_ESC                 (0x01)
  #define KEY_BACKSPACE           (0x0E)
  #define KEY_TAB                 (0x0F)
  #define KEY_ENTER               (0x1C)
  #define KEY_LCTRL             (0x1D)
  #define KEY_LSHIFT              (0x2A)
  #define KEY_RSHIFT              (0x36)

  #define KEY_LALT                 (0x38)
  #define KEY_SPACE               (0x39)
  #define KEY_CAPSLOCK            (0x3A)
  #define KEY_NUMLOCK             (0x45)
  #define KEY_SCROLLLOCK          (0x46)
  #define KEY_SYSREQ              (0x54)
  #define KEY_F11                 (0x57)
  #define KEY_F12                 (0x58)

  // note !!! the following keys are defined by Cybertek inc. and are NOT
  //          standard scancodes accesible via port 60h,
  //          only available on an AT keyboard.

  #define KEY_RALT     (0x59)
  #define KEY_RCTRL (0x5A)
  #define KEY_PAUSE (0x5B)
  #define KEY_INS (0x5C)
  #define KEY_DEL (0x5D)
  #define KEY_HOME (0x5E)
  #define KEY_END (0x5F)
  #define KEY_PGUP (0x60)
  #define KEY_PGDN (0x61)
  #define KEY_LEFT (0x62)
  #define KEY_RIGHT (0x63)
  #define KEY_UP (0x64)
  #define KEY_DOWN (0x65)
  #define KEY_PRTSC (0x66)
  #define KEY_BREAK (0x67)

  ///////////////////////////////////////////////////////////////

  #define KEY_BIOS_ESC                (27)
  #define KEY_BIOS_ENTER              (13)
  #define KEY_BIOS_BACKSPACE          (8)
  #define KEY_BIOS_TAB                (9)

  #define KEY_BIOS_PGDN               ('Q' << 8)
  #define KEY_BIOS_PGUP               ('I' << 8)
  #define KEY_BIOS_UP                 ('H' << 8)
  #define KEY_BIOS_LEFT               ('K' << 8)
  #define KEY_BIOS_DEL                (83 << 8)
  #define KEY_BIOS_INS                (0x100+82)
  #define KEY_BIOS_DOWN               ('P' << 8)
  #define KEY_BIOS_RIGHT              (0x100+'M')
  #define KEY_BIOS_HOME               (0x100+'G')
  #define KEY_BIOS_END                (0x100+'O')
  #define KEY_BIOS_F1                 (0x100+59)
  #define KEY_BIOS_F10                (0x100+68)
  #define KEY_BIOS_SHIFTTAB           (0x100+0xF)

  #define KEY_BIOS__ABORT (-1)
  #define KEY__ABORT (-1)

  #ifdef __cplusplus
  }
  #endif


#endif
