

#include <mem.h>
#include <dos.h>

#include "lin.h"
#include "dpmi.h"
#include "cgdk.h"

#if (defined __TURBOC__) || (defined CGDK_TRIAL)

void LIN_memcpy (void *dst, void *spr, int count)
{
  int i, j;

  for (j = 0; j < 3; j++)
  {
    for (i = 0; i < count; i++)
    {
      if (((char *)dst) [i] != ((char *)spr) [i])
      {
        ((char *)dst) [i] = ((char *)spr) [i];
      }
    }
  }
}


void LIN_sprcpy (void *dst, void *spr, int count)
{
  int i, j;

  for (j = 0; j < 3; j++)
  {
    for (i = 0; i < count; i++)
    {
      if (((char *)spr) [i])
      {
        if (((char *)dst) [i] != ((char *)spr) [i])
        {
          ((char *)dst) [i] = ((char *)spr) [i];
        }
      }
    }
  }
}

void LIN_movSpr_ (void *dst, int dstFormatX,
                 void *spr, int sprFormatX, int sprSizeX,int sprSizeY)
{
  for ( ; sprSizeY > 0; sprSizeY--)
  {
    LIN_sprcpy (dst, spr, sprSizeX);
    * (int *) &dst  += dstFormatX;
    * (int *) &spr += sprFormatX;
  }
}

void LIN_movWin_ (void *dst, int dstFormatX,
                 void *spr, int sprFormatX, int sprSizeX,int sprSizeY)
{
  for ( ; sprSizeY > 0; sprSizeY--)
  {
    LIN_memcpy (dst, spr, sprSizeX);
    * (int *) &dst  += dstFormatX;
    * (int *) &spr += sprFormatX;
  }
}


#endif


void LIN_memset_ (void *dst, int dstFormatX, int value,
                  int sprSizeX, int sprSizeY)
{

  int ds;

  ds = DPMI_getDS ();
  DPMI_setDS (DPMI_getES ());
  for ( ; sprSizeY > 0; sprSizeY--)
  {
    memset (dst, value, sprSizeX);
    * (int *) &dst += dstFormatX;
  }
  DPMI_setDS (ds);
}


void LIN_movWin (void *dst, int x, int y, int dstFormatX, int dstFormatY,
                 void *spr, int sprFormatX, int sprSizeX, int sprSizeY)
{
  int sizeX;
  int sizeY;
  int startX;
  int startY;

  startX = (x > 0) ? 0 : -x;
  startY = (y > 0) ? 0 : -y;

  if ((startX < sprSizeX) && (startY < sprSizeY))
  {
    sizeX = ((x + sprSizeX) <= dstFormatX) ? sprSizeX : dstFormatX - x;
    sizeY = ((y + sprSizeY) <= dstFormatY) ? sprSizeY : dstFormatY - y;

    if (sizeX > 0)
    {
      LIN_movWin_ ((char *)dst+(x+startX)+(y+startY)*dstFormatX, dstFormatX,
                     (char *)spr+startX+startY*sprFormatX, sprFormatX,
                      sizeX-startX, sizeY-startY);
    }
  }
}


void LIN_movSpr (void *dst, int x, int y, int dstFormatX, int dstFormatY,
                 void *spr, int sprFormatX, int sprSizeX, int sprSizeY)
{
  int sizeX;
  int sizeY;
  int startX;
  int startY;

  startX = (x > 0) ? 0 : -x;
  startY = (y > 0) ? 0 : -y;

  if ((startX < sprSizeX) && (startY < sprSizeY))
  {
    sizeX = ((x + sprSizeX) <= dstFormatX) ? sprSizeX : dstFormatX - x;
    sizeY = ((y + sprSizeY) <= dstFormatY) ? sprSizeY : dstFormatY - y;

    if (sizeX > 0)
    {
      LIN_movSpr_ ((char *)dst+(x+startX)+(y+startY)*dstFormatX, dstFormatX,
                     (char *)spr+startX+startY*sprFormatX, sprFormatX,
                      sizeX-startX, sizeY-startY);
    }
  }
}


void LIN_get (void *dst,int x, int y, int dstFormatX, int dstFormatY,
              void *spr, int sprFormatX, int sprSizeX, int sprSizeY)
{
  int sizeX;
  int sizeY;
  int startX;
  int startY;

  startX = (x > 0) ? 0 : -x;
  startY = (y > 0) ? 0 : -y;

  if ((startX < sprSizeX) && (startY < sprSizeY))
  {
    sizeX = ((x + sprSizeX) <= dstFormatX) ? sprSizeX : dstFormatX - x;
    sizeY = ((y + sprSizeY) <= dstFormatY) ? sprSizeY : dstFormatY - y;

    if (sizeX > 0)
    {
      LIN_movWin_ ((char *)spr+startX+startY*sprFormatX, sprFormatX,
                   (char *)dst+(x+startX)+(y+startY)*dstFormatX, dstFormatX,
                    sizeX-startX, sizeY-startY);
    }
  }
}

void LIN_memset (void *dst, int x, int y, int dstFormatX, int dstFormatY,
                 int value, int sprSizeX, int sprSizeY)
{
  int sizeX;
  int sizeY;
  int startX;
  int startY;

  startX = (x > 0) ? 0 : -x;
  startY = (y > 0) ? 0 : -y;

  if ((startX < sprSizeX) && (startY < sprSizeY))
  {
    sizeX = ((x + sprSizeX) <= dstFormatX) ? sprSizeX : dstFormatX - x;
    sizeY = ((y + sprSizeY) <= dstFormatY) ? sprSizeY : dstFormatY - y;

    if (sizeX > 0)
    {
      LIN_memset_ ((char *)dst+(x+startX)+(y+startY)*dstFormatX, dstFormatX,
                    value, sizeX-startX, sizeY-startY);
    }
  }
}
