
#include <stdio.h>
#include <stdlib.h>

#include "lzw.h"

static int LZWM_inLimit;
static int LZWM_inCur;
static int LZWM_outCur;

static char *LZWM_inSector;
static char *LZWM_outSector;

void LZWM_init (TLZW *lzw, const void *inBuf, long len, void *outBuf)
{
  LZWM_inLimit = len;
  LZWM_inCur = 0;
  LZWM_outCur = 0;
  LZWM_inSector = (char *) inBuf;
  LZWM_outSector = (char *) outBuf;

  lzw->putChar = LZWM_putChar;
  lzw->getChar = LZWM_getChar;
}

int LZWM_getChar (void)
{
  int result;

  if (LZWM_inCur == LZWM_inLimit)
  {
    return EOF;
  }

  result = (unsigned) LZWM_inSector [LZWM_inCur];
  LZWM_inCur++;
  return result;
}

int LZWM_putChar (int c)
{
  LZWM_outSector [LZWM_outCur] = c;
  LZWM_outCur++;

  return c;
}

void LZWM_done (TLZW *lzw)
{
  lzw->putChar = LZW_putCharError;
  lzw->getChar = LZW_getCharError;
}


int LZWM_compress (void *buf, long len, void *lzwBuf)
{
  TLZW *lzw;

  lzw = LZW_new ();
  if (lzw == NULL)
    return 0;

  LZW_compressInit (lzw);
  LZWM_init (lzw, buf, len, lzwBuf);
  LZW_run (lzw);

  len = lzw->bytesWritten;

  LZWM_done (lzw);
  LZW_free (lzw);

  return len;
}

int LZWM_uncompress (void *lzwBuf, long len, void *buf)
{
  TLZW *lzw;

  lzw = LZW_new ();
  if (lzw == NULL)
    return 0;

  LZW_uncompressInit (lzw);
  LZWM_init (lzw, lzwBuf, len, buf);
  LZW_run (lzw);

  len = lzw->bytesWritten;

  LZWM_done (lzw);
  LZW_free (lzw);

  return len;
}
