
#include <stdlib.h>
#include <math.h>

#include "cgdk.h"
#include "svga.h"
#include "l256_.h"
#include "lin.h"
#include "timer.h"
#include "dpmi.h"

#define X256_MOVWIN2 0
#define X256_MOVSPR2 1
#define X256_MOVWIN 2
#define X256_MOVSPR 3

extern void cdecl X256_movPlane (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_getPlane (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlane2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlaneSpr (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlaneSpr2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX);


struct TX256C
{
  int  plane;
  void *dst;
  void *dst2;
  void *src;
  short  formatX;
  short  sizeX;
  short  sizeY;
  short  type;
};

static TX256C *X256C;
static int X256C_last;
static int X256C_maxCached;

void X256C_done (void)
{
  if (X256C)
  {
    _free (X256C);
    X256C = NULL;
  }
  X256C_maxCached = 0;
}

void X256C_init (void)
{
  X256C_done ();
  X256C_maxCached = 4*SVGAC_maxCached;
  X256C = (TX256C *) _alloc (X256C_maxCached * sizeof (TX256C));
  X256C_last = 0;
}

void X256C_putPlaneSpr (int plane, void *dst, void *src,
                        int sizeX, int sizeY, int formatX)
{
  int pos;

  if (sizeY)
  {
    pos = X256C_last;
    X256C_last++;
    X256C [pos].plane = plane;
    X256C [pos].dst = dst;
    X256C [pos].src = src;
    X256C [pos].formatX = formatX;
    X256C [pos].sizeX = sizeX;
    X256C [pos].sizeY = sizeY;
    X256C [pos].type = X256_MOVSPR;
  }
}

void X256C_putPlane (int plane, void *dst, void *src,
                     int sizeX, int sizeY, int formatX)
{
  int pos;

  if (sizeY)
  {
    pos = X256C_last;
    X256C_last++;
    X256C [pos].plane = plane;
    X256C [pos].dst = dst;
    X256C [pos].src = src;
    X256C [pos].formatX = formatX;
    X256C [pos].sizeX = sizeX;
    X256C [pos].sizeY = sizeY;
    X256C [pos].type = X256_MOVWIN;
  }
}

void X256C_putPlaneSpr2 (int plane, void *dst, void *dst2, void *src,
                        int sizeX, int sizeY, int formatX)
{
  int pos;

  if (sizeY)
  {
    pos = X256C_last;
    X256C_last++;
    X256C [pos].plane = plane;
    X256C [pos].dst = dst;
    X256C [pos].dst2 = dst2;
    X256C [pos].src = src;
    X256C [pos].formatX = formatX;
    X256C [pos].sizeX = sizeX;
    X256C [pos].sizeY = sizeY;
    X256C [pos].type = X256_MOVSPR2;
  }
}

void X256C_putPlane2 (int plane, void *dst, void *dst2, void *src,
                     int sizeX, int sizeY, int formatX)
{
  int pos;

  if (sizeY)
  {
    pos = X256C_last;
    X256C_last++;
    X256C [pos].plane = plane;
    X256C [pos].dst = dst;
    X256C [pos].dst2 = dst2;
    X256C [pos].src = src;
    X256C [pos].formatX = formatX;
    X256C [pos].sizeX = sizeX;
    X256C [pos].sizeY = sizeY;
    X256C [pos].type = X256_MOVWIN2;
  }
}




static void X256C_flushMov (TX256C *xc)
{
  XVGA_setWritePlane (xc->plane);

  switch (xc->type)
  {
    #ifndef __TURBOC__
    case X256_MOVWIN2:
    {
      X256_movPlane2 (xc->dst, xc->dst2, xc->src, xc->sizeX, xc->sizeY, xc->formatX);
      break;
    }
    case X256_MOVSPR2:
    {
      X256_movPlaneSpr2 (xc->dst, xc->dst2, xc->src, xc->sizeX, xc->sizeY, xc->formatX);
      break;
    }
    #endif
    case X256_MOVWIN:
    {
      X256_movPlane (xc->dst, xc->src, xc->sizeX, xc->sizeY, xc->formatX);
      break;
    }
    case X256_MOVSPR:
    {
      X256_movPlaneSpr (xc->dst, xc->src, xc->sizeX, xc->sizeY, xc->formatX);
      break;
    }
  }
}

void X256C_flush2 (void)
{
/*  int i;
  int pos;
  int retrace;
  int trap;
  int endY;
  int beamY;
  TX256C *xc;
  double retraceRest;
  int retraceTime;

  xc = X256C;

  retrace = TIMER_retraceTime / TIMER_retraceTimeMaxF;
  CGDK_fp++;

  i = 0;
  while (retrace == ((int) (TIMER_retraceTime / TIMER_retraceTimeMaxF)))
  {
    CGDK_fp++;
    retraceRest = fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF);
    CGDK_fp+=2;
    if (retraceRest > TIMER_retraceTimeMaxF09)
    {
      SVGA_wait (1);
      break;
    }

    if (i == X256C_last)
    {
      SVGA_wait (1);
      break;
    }

    beamY = (SVGA_maxY * (retraceRest-TIMER_retraceTimeMaxF01)) / TIMER_retraceTimeMaxF;
    CGDK_fp+=2;

    retraceTime = TIMER_retraceTime;
    while (TIMER_retraceTime == retraceTime)
    {
      if (i == X256C_last)
        break;

      pos = (long) xc->dst / XVGA_columns + xc->sizeY;
      if (pos < SVGA_curY)
      {
        pos = (long) xc->dst2 / XVGA_columns + xc->sizeY;
      }

      endY = pos - SVGA_curY;


      if (endY < beamY)
      {
        X256C_flushMov (xc);
        *(int *) &xc += sizeof (TX256C);
        i++;
      }
      else
      {
        if (SVGA_retraceDetect ())
        {
          break;
        }
      }
    }
    if (i == X256C_last || SVGA_retraceDetect ())
    {
      SVGA_wait (1);
      break;
    }
  }

  SVGA_scroll ();

//  for ( ; i < X256C_last; i++)
  {
    X256C_flushMov (xc);
    *(int *) &xc += sizeof (TX256C);
  }

  X256C_last = 0;
*/

  int i;
  int retraceTime;
  int retraceDetect;
  long beamY;
  long beamMaxY;
  long end1Y, end2Y;
//  long end1X, end2X;
  TX256C *lc;
  int es;
  double retraceRest;
  double beamRest;

  lc = X256C;

  i = 0;

/*
  CGDK_fp+=2;

  retraceTime = TIMER_retraceTime;
  retraceDetect = 0;
  while (retraceRest = fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF),
         retraceRest < 0.95*TIMER_retraceTimeMaxF)
  {
    beamRest = (retraceRest-0.1*TIMER_retraceTimeMaxF) / TIMER_retraceTimeMaxF;
    beamY = SVGA_curYPrev + SVGA_maxY * beamRest;
    beamMaxY = SVGA_curYPrev + SVGA_maxY;
    CGDK_fp+=5;

    retraceTime = TIMER_retraceTime;
    while ((TIMER_retraceTime == retraceTime) && (i < X256C_last) &&
           (retraceDetect = SVGA_retraceDetect (), !retraceDetect) )
    {
      end1Y = ((long)lc->dst) / XVGA_columns + lc->sizeY;
      end2Y = ((long)lc->dst2) / XVGA_columns + lc->sizeY;

      if ( ((end1Y < beamY) || (end1Y > beamMaxY)) &&
           ( (lc->type == X256_MOVWIN) || (lc->type == X256_MOVSPR)||(end2Y < beamY) || (end2Y > beamMaxY)
           )
         )
      {
        X256C_flushMov (lc);
        *(int *) &lc += sizeof (TX256C);
        i++;
      }
    }
    if (i == X256C_last || retraceDetect)
    {
      break;
    }
  }

  if (i == X256C_last)
  {
    SVGA_wait (1);
  }
  else
  {
    retraceTime = TIMER_retraceTime;
    while (TIMER_retraceTime - retraceTime < (0.1*TIMER_retraceTimeMaxF))
    {
      if (SVGA_retraceDetect ())
      {
        SVGA_wait (1);
        break;
      }
    }
  }
*/
  SVGA_wait (1);
  SVGA_scroll ();

  for ( ; i < X256C_last; i++)
  {
    X256C_flushMov (lc);
    *(int *) &lc += sizeof (TX256C);
  }

  X256C_last = 0;
}

void X256C_flush (void)
{
  int i;
  long beamY;
  long beamMaxY;
  long end1Y, end2Y;
  TX256C *lc;
  long videoBuf;

  i = 0;
  lc = X256C;

  if (SVGA_enableRetrace)
  {
    videoBuf = (long) SVGA_videoBuf;

    beamY = SVGA_curY + SVGA_maxY/3;
    beamMaxY = SVGA_curY + SVGA_maxY;

    while (fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF) < 0.666*TIMER_retraceTimeMaxF)
      ;

    while (i < X256C_last)
    {
      if ((lc->type == X256_MOVWIN) || (lc->type == X256_MOVSPR))
      {
	end1Y = (((long)lc->dst-videoBuf) / XVGA_columns + lc->sizeY);
	if (SVGA_curY > SVGA_maxY)
	{
	  if (end1Y < SVGA_curY)
	  {
	    end1Y += SVGA_virtualY;
	  }
	}

	if ( (end1Y < beamY) || (end1Y > beamMaxY) )
	{
	  X256C_flushMov (lc);
	  *(int *) &lc += sizeof (TX256C);
	  i++;
	}
	else
	{
	  break;
	}

      }
      else
      {
	end1Y = ((long)lc->dst-videoBuf) / XVGA_columns + lc->sizeY;
	end2Y = ((long)lc->dst2-videoBuf) / XVGA_columns + lc->sizeY;

	if ( ((end1Y < beamY) || (end1Y > beamMaxY)) &&
	       ( (end2Y < beamY) || (end2Y > beamMaxY) )
	     )
	{
	  X256C_flushMov (lc);
	  *(int *) &lc += sizeof (TX256C);
	  i++;
	}
	else
	{
	  break;
	}
      }
    }
  }

  SVGA_wait (1);
//  SVGA_scroll ();

  for ( ; i < X256C_last; i++)
  {
    X256C_flushMov (lc);
    *(int *) &lc += sizeof (TX256C);
  }

  X256C_last = 0;
}
